// Este programa implementa la bsqueda en profundidad (iterativa) sobre una grfica almacenada en listas de adyacencia, imprimiendo los vrtices conforme son visitados
// Adems de la informacin de la grfica, este programa pide el ndice del vrtice inicial del recorrido
#include <iostream>
#include <stack>
#include <vector>
using namespace std;

void profundidad(int i, vector<int> adyacencia[], int v)
{  // invariante: lo marcamos como visitado al tiempo que lo imprimimos
   vector<bool> visitados(v, false);
   stack<int> pila;
   pila.push(i);                       // metemos en la pila al vrtice inicial

   do {
      int actual = pila.top( );
      pila.pop( );
      if (visitados[actual]) {         // si ya lo imprimimos, lo ignoramos
         continue;
      }

      cout << actual << "\n";          // lo imprimimos
      visitados[actual] = true;        // lo marcamos
      for (int vecino : adyacencia[actual]) {         // iterativamente metemos a sus vecinos en la pila
         pila.push(vecino);                           // los vecinos listados primero se metern primero en la pila, pero saldrn en orden contrario (primero el ltimo vecino listado)
      }                                               // este orden es diferente al orden de profundidad recursiva, pero ambos son rdenes en profunidad vlidos
   } while (!pila.empty( ));
}

int main( )
{
   int v, e;
   cin >> v >> e;

   vector<int> adyacencia[v];
   for (int i = 0; i < e; ++i) {
      int x, y;
      cin >> x >> y;
      adyacencia[x].push_back(y);
      adyacencia[y].push_back(x);
   }

   int inicial;
   cin >> inicial;
   profundidad(inicial, adyacencia, v);
}
