// Este programa implementa la bsqueda en amplitud (iterativa) sobre una grfica almacenada en listas de adyacencia, imprimiendo los vrtices conforme son visitados
// Adems de la informacin de la grfica, este programa pide el ndice del vrtice inicial del recorrido
// Esta versin de bsqueda en amplitud marca los vrtices como visitados de manera anticipada
#include <iostream>
#include <queue>
#include <vector>
using namespace std;

void amplitud(int i, vector<int> adyacencia[], int v)
{  // invariante: lo marcamos en cuanto lo metemos a la cola; esto es correcto, pues los vrtices se imprimirn justo en el orden en el que se meten a la cola
   vector<bool> visitados(v, false);
   queue<int> cola;
   cola.push(i);                       // metemos en la cola al vrtice inicial
   visitados[i] = true;                // lo marcamos

   do {
      int actual = cola.front( );
      cola.pop( );
      cout << actual << "\n";          // como un elemento que entra a la cola ya est marcado, cuando sale lo imprimimos sin preguntar si estaba marcado (eso ya sabemos que es cierto)

      for (int vecino : adyacencia[actual]) {      // iterativamente inspeccionamos a sus vecinos
         if (!visitados[vecino]) {                 // si no est marcado
            visitados[vecino] = true;              // lo marcamos
            cola.push(vecino);                     // lo metemos a la cola
         }
      }
   } while (!cola.empty( ));
}

int main( )
{
   int v, e;
   cin >> v >> e;

   vector<int> adyacencia[v];
   for (int i = 0; i < e; ++i) {
      int x, y;
      cin >> x >> y;
      adyacencia[x].push_back(y);
      adyacencia[y].push_back(x);
   }

   int inicial;
   cin >> inicial;
   amplitud(inicial, adyacencia, v);
}
