// Este programa implementa la bsqueda en amplitud sobre una grfica almacenada en listas de adyacencia para calcular las distancias mnimas de un origen al resto de los vrtices
// Adems de la informacin de la grfica, este programa pide el ndice del vrtice inicial del recorrido
// Las aristas no tienen costo: se asume que son de costo unitario
#include <iostream>
#include <queue>
#include <vector>
using namespace std;

vector<int> calcula_distancias(int i, vector<int> adyacencia[], int v)
{
   vector<int> distancias(v, -1);        // para vrtices que no han sido encontrados, diremos que su distancia es -1 del origen (lo cual claramente es imposible)
   queue<int> cola;
   cola.push(i);                          // metemos en la cola al vrtice inicial
   distancias[i] = 0;                     // y decimos que su distancia es la 0

   do {
      int actual = cola.front( );
      cola.pop( );

      for (int vecino : adyacencia[actual]) {               // iterativamente inspeccionamos a sus vecinos
         if (distancias[vecino] == -1) {                    // si an no habamos visto ese vrtice, su distancia es -1
            distancias[vecino] = distancias[actual] + 1;    // en ese caso, su distancia es uno ms que la distancia que nosotros tenemos del origen
            cola.push(vecino);                              // y lo metemos a la cola
         }
      }
   } while (!cola.empty( ));

   return distancias;
}

int main( )
{
   int v, e;
   cin >> v >> e;

   vector<int> adyacencia[v];
   for (int i = 0; i < e; ++i) {
      int x, y;
      cin >> x >> y;
      adyacencia[x].push_back(y);
      adyacencia[y].push_back(x);
   }

   int inicial;
   cin >> inicial;
   auto distancias = calcula_distancias(inicial, adyacencia, v);

   for (int i = 0; i < v; ++i) {                         // imprimimos las distancias
      cout << i << ": " << distancias[i] << "\n";        // vrtices inalcanzables seguirn teniendo -1 como distancia
   }
}
